C*PGPOLY -- draw a polygon, using fill-area attributes
C%void cpgpoly(int n, const float *xpts, const float *ypts);
C+
      SUBROUTINE CLIPPOLY (N, XPTS, YPTS, QX, QY, RX, RY, 
     -                   MAXOUT, NUMOUT,
     -                   XL, XH, YL, YH)
      INTEGER N
      REAL XPTS(100), YPTS(100)
      REAL    QX(100), QY(100), RX(100), RY(100)
C
C Fill-area primitive routine: shade the interior of a closed
C polygon in the current window.  The action of this routine depends
C on the setting of the Fill-Area Style attribute (see PGSFS).
C The polygon is clipped at the edge of the
C window. The pen position is changed to (XPTS(1),YPTS(1)) in world
C coordinates (if N > 1).  If the polygon is not convex, a point is
C assumed to lie inside the polygon if a straight line drawn to
C infinity intersects and odd number of the polygon's edges.
C
C Arguments:
C  N      (input)  : number of points defining the polygon; the
C                    line consists of N straight-line segments,
C                    joining points 1 to 2, 2 to 3,... N-1 to N, N to 1.
C                    N should be greater than 2 (if it is 2 or less,
C                    nothing will be drawn).
C  XPTS   (input)  : world x-coordinates of the vertices.
C  YPTS   (input)  : world y-coordinates of the vertices.
C                    Note: the dimension of arrays XPTS and YPTS must be
C                    greater than or equal to N.
C  QX, QY, RX, RY    Working arrays size MAXOUT. 
C                    Results in QX, QY
C  NUMOUT            Num rexults coords
C  XL..YH            Clipping window
C--
C 21-Nov-1983 - [TJP].
C 16-Jul-1984 - revised to shade polygon with GRFA [TJP].
C 21-Oct-1985 - test PGFAS [TJP].
C 25-Nov-1994 - implement clipping [TJP].
C 13-Jan-1994 - fix bug in clipping [TJP].
C  6-Mar-1995 - add support for fill styles 3 and 4 [TJP].
C 12-Sep-1995 - fix another bug in clipping [TJP].
C MARCH 00 - OZAPI VERSION - MOST TESTING DONE BEFORE CALL
C-----------------------------------------------------------------------

      INTEGER I, N1, N2, N3, N4
      REAL    XL, XH, YL, YH
CC      INCLUDE 'pgplot.inc'
C
C
C  FIRST SEE IF COMPLETELY INSIDE CLIPPING RECT
C
      DO 500 L=1,N
        IF (XPTS(L).LT.XL)  GO TO 1000
        IF (XPTS(L).GT.XH)  GO TO 1000
        IF (YPTS(L).LT.YL)  GO TO 1000
        IF (YPTS(L).GT.YH)  GO TO 1000
        RX(L) = XPTS(L)
        RY(L) = YPTS(L)
500   CONTINUE
      NUMOUT = N
      RETURN
C     
C clipping required: the vertices of the clipped
C polygon are put in temporary arrays QX,QY, RX, RY.
C  
1000  CONTINUE   
      CALL GRPOCL(N,  XPTS, YPTS, 1, XL, MAXOUT, N1, QX, QY)
      IF (N1.GT.MAXOUT) GOTO 50
      IF (N1.LT.3) GOTO 50
      CALL GRPOCL(N1, QX,   QY,   2, XH, MAXOUT, N2, RX, RY)
      IF (N2.GT.MAXOUT) GOTO 50
      IF (N2.LT.3) GOTO 50
      CALL GRPOCL(N2, RX,   RY,   3, YL, MAXOUT, N3, QX, QY)
      IF (N3.GT.MAXOUT) GOTO 50
      IF (N3.LT.3) GOTO 50
      CALL GRPOCL(N3, QX,   QY,   4, YH, MAXOUT, N4, RX, RY)
      IF (N4.GT.MAXOUT) GOTO 50
      IF (N4.LT.3) GOTO 50
      NUMOUT = N4
      IF (RX(1).NE.RX(NUMOUT).OR.RY(1).NE.RY(NUMOUT)) THEN
        NUMOUT = NUMOUT + 1
        RX(NUMOUT) = RX(1)
        RY(NUMOUT) = RY(1)
      END IF
      RETURN

 50   CONTINUE
      NUMOUT = 0
      RETURN
C
      END


C  OZAPI

      SUBROUTINE CLIPLINE (N, XPTS, YPTS, QX, QY, 
     -                   MAXOUT, NUMOUT,
     -                   XL, XH, YL, YH)
      INTEGER N
      REAL XPTS(100), YPTS(100)
      REAL    QX(100), QY(100)
C
C Clip poly-line
C
C Arguments:
C  N      (input)  : number of points defining the polygon; the
C                    line consists of N straight-line segments,
C                    joining points 1 to 2, 2 to 3,... N-1 to N, N to 1.
C                    N should be greater than 2 (if it is 2 or less,
C                    nothing will be drawn).
C  XPTS   (input)  : world x-coordinates of the vertices.
C  YPTS   (input)  : world y-coordinates of the vertices.
C                    Note: the dimension of arrays XPTS and YPTS must be
C                    greater than or equal to N.
C  QX, QY            Result arrays size MAXOUT. 
C  NUMOUT            Num reSults coords
C  XL..YH            Clipping window
C--

      LOGICAL VIS

C
C  CLIP LINE
C
1000  CONTINUE
      NUMOUT = 0
      DO 2000 L=1,N-1
        X0 = XPTS(L)
        Y0 = YPTS(L)
        X1 = XPTS(L+1)
        Y1 = YPTS(L+1)
        CALL GRCLPL(X0, Y0, X1, Y1, VIS, XL, XH, YL, YH)
        IF (VIS)  THEN
          IF (NUMOUT.EQ.0) THEN
             QX(1) = X0
             QY(1) = Y0
             QX(2) = X1
             QY(2) = Y1
             NUMOUT = 2
          ELSE
             IF (X0.NE.QX(NUMOUT) .OR. Y0.NE.QY(NUMOUT)) THEN
               NUMOUT = NUMOUT + 1
               QX(NUMOUT) = X0
               QY(NUMOUT) = Y0
             END IF
             NUMOUT = NUMOUT + 1
             QX(NUMOUT) = X1
             QY(NUMOUT) = Y1
          END IF
          IF (NUMOUT.GT.MAXOUT-2)  RETURN
        END IF
2000  CONTINUE

C     
      RETURN
      END
